@@ -74,8 +74,8 @@ pgf.DEFAULT_FONT_V_CHAR = 308
 
 pgf.STYLE_FILE_BASENAME = "gnuplot-lua-tikz"  -- \usepackage{gnuplot-lua-tikz}
 
-pgf.REVISION = "107"
-pgf.REVISION_DATE = "2018/06/29 00:58:15"
+pgf.REVISION = "118"
+pgf.REVISION_DATE = "2023/04/12 19:21:00"
 
 pgf.styles = {}
 
@@ -252,7 +252,10 @@ pgf.write_doc_end = function()
 end
 
 pgf.write_graph_begin = function (font, noenv)
-  local global_opt = "" -- unused
+  local global_opt = ""
+  if gfx.opt.butt then
+    global_opt = ",cap=butt,join=miter"
+  end
   if gfx.opt.full_doc then
     gp.write(gfx.format[gfx.opt.tex_format].beforetikzpicture)
   end
@@ -366,7 +369,11 @@ end
 
 
 pgf.draw_points = function(t, pm)
-  gp.write("\\gppoint{"..pm.."}{")
+  local style_options = ""
+  if gfx.opacity < 1.0 then
+    style_options = string.format(",opacity=%.3f", gfx.opacity)
+  end
+  gp.write("\\gp3point{"..pm.."}{"..style_options.."}{")
   for i,v in ipairs(t) do
       gp.write("("..pgf.format_coord(v[1], v[2])..")")
   end
@@ -424,6 +431,10 @@ end
 pgf.draw_fill = function(t, pattern, color, saturation, opacity)
   local fill_path = ''
   local fill_style = color
+
+  if #t < 2 then
+    return
+  end
   
   if saturation < 100 then
     fill_style = fill_style .. ",color=.!"..saturation;
@@ -489,7 +500,8 @@ pgf.draw_raw_rgb_image = function(t, m, n, ll, ur, xfile)
   local ys = sf("%.3f", pgf.transform_ycoord(ur[2]) - pgf.transform_ycoord(ll[2]))
   gw("\\def\\gprawrgbimagedata{%\n  ")
   for cnt = 1,#t do
-    gw(sf("%02x%02x%02x", 255*t[cnt][1]+0.5, 255*t[cnt][2]+0.5, 255*t[cnt][3]+0.5))
+    gw(sf("%02x%02x%02x", math.floor(255*t[cnt][1]+0.5),
+          math.floor(255*t[cnt][2]+0.5), math.floor(255*t[cnt][3]+0.5)))
     if (cnt % 16) == 0 then
       gw("%\n  ")
     end
@@ -557,7 +569,7 @@ pgf.write_variables = function(t)
   end
 end
 
--- write style to seperate file, or whatever...
+-- write style to separate file, or whatever...
 pgf.create_style = function()
   local name_common  = pgf.STYLE_FILE_BASENAME.."-common.tex"
   local name_latex   = pgf.STYLE_FILE_BASENAME..".sty"
@@ -660,40 +672,30 @@ f:write([[
 \fi}
 \expandafter\gpchecktikzversion\pgfversion\relax
 
-% FIXME: is there a more elegant way to determine the output format?
+\def\gnuplot@output@ps{ps} % required for comparison in \gprawimage
 
-\def\pgfsysdriver@a{pgfsys-dvi.def}       % ps
-\def\pgfsysdriver@b{pgfsys-dvipdfm.def}   % pdf
-\def\pgfsysdriver@c{pgfsys-dvipdfmx.def}  % pdf
-\def\pgfsysdriver@d{pgfsys-dvips.def}     % ps
-\def\pgfsysdriver@e{pgfsys-pdftex.def}    % pdf
-\def\pgfsysdriver@f{pgfsys-tex4ht.def}    % html
-\def\pgfsysdriver@g{pgfsys-textures.def}  % ps
-\def\pgfsysdriver@h{pgfsys-vtex.def}      % ps
-\def\pgfsysdriver@i{pgfsys-xetex.def}     % pdf
+\expandafter\def\csname gnuplot@select@driver@pgfsys-dvi.def\endcsname     {ps}
+\expandafter\def\csname gnuplot@select@driver@pgfsys-dvipdfm.def\endcsname {pdf}
+\expandafter\def\csname gnuplot@select@driver@pgfsys-dvipdfmx.def\endcsname{pdf}
+\expandafter\def\csname gnuplot@select@driver@pgfsys-dvips.def\endcsname   {ps}
+\expandafter\def\csname gnuplot@select@driver@pgfsys-pdftex.def\endcsname  {pdf}
+\expandafter\def\csname gnuplot@select@driver@pgfsys-luatex.def\endcsname  {pdf}
+\expandafter\def\csname gnuplot@select@driver@pgfsys-tex4ht.def\endcsname  {html}
+\expandafter\def\csname gnuplot@select@driver@pgfsys-textures.def\endcsname{ps}
+\expandafter\def\csname gnuplot@select@driver@pgfsys-vtex.def\endcsname    {ps}
+\expandafter\def\csname gnuplot@select@driver@pgfsys-xetex.def\endcsname   {pdf}
 
-\newif\ifgppdfout\gppdfoutfalse
-\newif\ifgppsout\gppsoutfalse
+\ifcsname gnuplot@select@driver@\pgfsysdriver\endcsname
+  \edef\gnuplot@output@mode{\csname gnuplot@select@driver@\pgfsysdriver\endcsname}
+\else
+  \errmessage{The driver \pgfsysdriver\space is not supported by gnuplot-lua-tikz}%
+\fi
 
-\ifx\pgfsysdriver\pgfsysdriver@a
-  \gppsouttrue
-\else\ifx\pgfsysdriver\pgfsysdriver@b
-  \gppdfouttrue
-\else\ifx\pgfsysdriver\pgfsysdriver@c
-  \gppdfouttrue
-\else\ifx\pgfsysdriver\pgfsysdriver@d
-  \gppsouttrue
-\else\ifx\pgfsysdriver\pgfsysdriver@e
-  \gppdfouttrue
-\else\ifx\pgfsysdriver\pgfsysdriver@f
-  % tex4ht
-\else\ifx\pgfsysdriver\pgfsysdriver@g
-  \gppsouttrue
-\else\ifx\pgfsysdriver\pgfsysdriver@h
-  \gppsouttrue
-\else\ifx\pgfsysdriver\pgfsysdriver@i
-  \gppdfouttrue
-\fi\fi\fi\fi\fi\fi\fi\fi\fi
+\ifcsname PackageWarning\endcsname
+  \let\gnuplot@packagewarning\PackageWarning
+\else
+  \def\gnuplot@packagewarning#1#2{\immediate\write-1{Package #1 Warning: #2}}
+\fi
 
 % uncomment the following lines to make font values "appendable"
 % and if you are really sure about that ;-)
@@ -734,19 +736,13 @@ f:write([[
   \pgfsysprotocol@literal{#6 >}%
 }
 \def\gp@rawimage@html#1#2#3#4#5#6{%
-% FIXME: print a warning message here
+  \gnuplot@packagewarning{gnuplot-lua-tikz}{%
+    \noexpand\gp@rawimage is not implemented for \gnuplot@output@mode\space output.%
+  }%
 }
 
-\ifgppdfout
-  \def\gp@rawimage{\gp@rawimage@pdf}
-\else
-  \ifgppsout
-    \def\gp@rawimage{\gp@rawimage@ps}
-  \else
-    \def\gp@rawimage{\gp@rawimage@html}
-  \fi
-\fi
-
+% Select \gp@rawimage depending on the output mode
+\expandafter\let\expandafter\gp@rawimage\csname gp@rawimage@\gnuplot@output@mode\endcsname
 
 \def\gploadimage#1#2#3#4#5{%
   \pgftext[left,bottom,x=#1cm,y=#2cm] {\pgfimage[interpolate=false,width=#3cm,height=#4cm]{#5}};%
@@ -756,10 +752,10 @@ f:write([[
   \def\gp@image@size{#1}%
 }
 
-\def\gp@rawimage@#1#2#3#4#5#6#7#8{
+\def\gp@rawimage@#1#2#3#4#5#6#7#8{%
   \tikz@scan@one@point\gp@set@size(#6,#7)\relax%
   \tikz@scan@one@point\pgftransformshift(#2,#3)\relax%
-  \pgftext {%
+  \pgftext{%
     \pgfsys@beginpurepicture%
     \gp@image@size% fill \pgf@x and \pgf@y
     \gp@rawimage{#1}{#4}{#5}{\pgf@x}{\pgf@y}{#8}%
@@ -771,18 +767,18 @@ f:write([[
 %% color model is 'cmyk' or 'rgb' (default)
 \def\gprawimage#1#2#3#4#5#6#7#8#9{%
   \ifx&#9&%
-    \gp@rawimage@{#1}{#2}{#3}{#4}{#5}{#6}{#7}{#8}
+    \gp@rawimage@{#1}{#2}{#3}{#4}{#5}{#6}{#7}{#8}%
   \else
-    \ifgppsout
-      \gp@rawimage@{#1}{#2}{#3}{#4}{#5}{#6}{#7}{#8}
+    \ifx\gnuplot@output@mode\gnuplot@output@ps
+      \gp@rawimage@{#1}{#2}{#3}{#4}{#5}{#6}{#7}{#8}%
     \else
-      \gploadimage{#2}{#3}{#6}{#7}{#9}
+      \gploadimage{#2}{#3}{#6}{#7}{#9}%
     \fi
   \fi
 }
 
 %
-% gnuplottex comapatibility
+% gnuplottex compatibility
 % (see http://www.ctan.org/tex-archive/help/Catalogue/entries/gnuplottex.html)
 %
 
@@ -829,6 +825,7 @@ f:write([[
 % prevent plot mark distortions due to changes in the PGF transformation matrix
 % use `\gpscalepointstrue' and `\gpscalepointsfalse' for enabling and disabling
 % point scaling
+% 3-parameter variant gp3point passes style options as well as point type and coordinates
 %
 \newif\ifgpscalepoints
 \tikzset{gp shift only/.style={%
@@ -837,6 +834,9 @@ f:write([[
 \def\gppoint#1#2{%
   \path[solid] plot[only marks,gp point,mark options={gp shift only},#1] coordinates {#2};%
 }
+\def\gp3point#1#2#3{%
+  \path[solid#2] plot[only marks,gp point,mark options={gp shift only},#1] coordinates {#3};%
+}
 
 
 %
@@ -883,11 +883,11 @@ f:write([[
 %  #3 coordinate of "north east"
 %
 \def\gpdefrectangularnode#1#2#3{%
-  \expandafter\gdef\csname pgf@sh@ns@#1\endcsname{rectangle}
+  \expandafter\gdef\csname pgf@sh@ns@#1\endcsname{rectangle}%
   \expandafter\gdef\csname pgf@sh@np@#1\endcsname{%
     \def\southwest{#2}%
     \def\northeast{#3}%
-  }
+  }%
   \pgfgettransform\pgf@temp%
   % once it is defined, no more transformations will be applied, I hope
   \expandafter\xdef\csname pgf@sh@nt@#1\endcsname{\pgf@temp}%
@@ -910,9 +910,9 @@ f:write([[
   line join=round,%
 }}
 
-\tikzset{gp node left/.style={anchor=mid west,yshift=-.12ex}}
-\tikzset{gp node center/.style={anchor=mid,yshift=-.12ex}}
-\tikzset{gp node right/.style={anchor=mid east,yshift=-.12ex}}
+\tikzset{gp node left/.style={anchor=mid west,yshift=-.12ex,line width=0pt}}
+\tikzset{gp node center/.style={anchor=mid,yshift=-.12ex,line width=0pt}}
+\tikzset{gp node right/.style={anchor=mid east,yshift=-.12ex,line width=0pt}}
 
 % basic plot mark size (points)
 \newdimen\gpbasems
@@ -1045,6 +1045,7 @@ pgf.print_help = function(fwrite)
       {nogppoints | gppoints}
       {picenvironment | nopicenvironment}
       {noclip | clip}
+      {butt}
       {notightboundingbox | tightboundingbox}
       {background "<colorpec>"}
       {size <x>{unit},<y>{unit}}
@@ -1095,6 +1096,9 @@ pgf.print_help = function(fwrite)
  is set. Neither a fixed bounding box nor a crop box is set if the
  'plotsize' or 'tightboundingbox' option is used.
 
+ 'butt' changes the linecap property to "butt" and the linejoin
+ property to "miter".  The defaults are "round" and "round".
+
  If 'tightboundingbox' is set the 'clip' option is ignored and the
  final bounding box is the natural bounding box calculated by tikz.
 
@@ -1104,7 +1108,7 @@ pgf.print_help = function(fwrite)
  sign ('#'). E.g. '#ff0000' specifies pure red. If omitted the
  background is transparent.
 
- The 'size' option expects two lenghts <x> and <y> as the canvas
+ The 'size' option expects two lengths <x> and <y> as the canvas
  size. The default size of the canvas is ]]..pgf.DEFAULT_CANVAS_SIZE_X..[[cm x ]]..pgf.DEFAULT_CANVAS_SIZE_Y..[[cm.
 
  The 'scale' option works similar to the 'size' option but expects
@@ -1151,13 +1155,13 @@ pgf.print_help = function(fwrite)
  for every linetype. The default plotstyle is 'smooth' for every
  linetype >= 1.
 
- By using the 'tikzarrows' option the gnuplot arrow styles defined by
- the user will be mapped to TikZ arrow styles. This is done by 'misusing'
- the angle value of the arrow definition. E.g. an arrow style with the
- angle '7' will be mapped to the TikZ style 'gp arrow 7' ignoring all the
- other given values. By default the TikZ terminal uses the stealth' arrow
- tips for all arrows. To obtain the default gnuplot behaviour please use
- the 'gparrows' option.
+ By default the tikz terminal produces simple LaTeX arrows.
+ To produce arrows in accord with gnuplot's 'arrowstyle' settings,
+ use the 'gparrows' option.  The 'tikzarrows' option is a third alternative
+ that bypasses both of these. Instead the arrowstyle 'angle' parameter is
+ used to index a set of 12 pre-defined TikZ arrow styles.
+ E.g. an arrow style with the angle '7' will be mapped to the TikZ style
+ 'gp arrow 7' ignoring all other arrowstyle settings.
 
  With 'cmykimages' the CMYK color model will be used for inline image data
  instead of the RGB model. All other colors (like line colors etc.) are
@@ -1167,7 +1171,7 @@ pgf.print_help = function(fwrite)
  By using the 'externalimages' option all bitmap images will be written
  as external PNG images and included at compile time of the document.
  Generating DVI and later postscript files requires to convert the PNGs
- into EPS files in a seperate step e.g. by using ImageMagick's `convert`.
+ into EPS files in a separate step e.g. by using ImageMagick's `convert`.
  Transparent bitmap images are always generated as an external PNGs.
 
  The 'nobitmap' option let images be rendered as filled rectangles instead
@@ -1175,7 +1179,7 @@ pgf.print_help = function(fwrite)
  images are externalized.
 
  The 'providevars' options makes gnuplot's internal and user variables
- available by using the '\gpgetvar{<var name>}' commmand within the TeX
+ available by using the '\gpgetvar{<var name>}' command within the TeX
  script. Use gnuplot's 'show variables all' command to see the list
  of valid variables.
 
@@ -1326,8 +1330,16 @@ gfx.format.latex = {
   docheader        = "\\documentclass["..pgf.DEFAULT_FONT_SIZE.."pt]{article}\n"
                       .."\\usepackage[T1]{fontenc}\n"
                       .."\\usepackage{textcomp}\n\n"
-                      .."\\usepackage[utf8x]{inputenc}\n"
-                      .."\\SetUnicodeOption{mathletters}\n\n"
+                      .."\\usepackage{ifluatex,ifxetex}\n"
+                      .."\\ifluatex\n"
+                      .."  \\usepackage{fontspec}\n  \\setmainfont{Free Serif}\n"
+                      .."\\else\\ifxetex\n"
+                      .."  \\usepackage{fontspec}\n  \\setmainfont{FreeSerif}\n"
+                      .."\\else\n"
+                      .."  \\usepackage{ucs}\n"
+                      .."  \\SetUnicodeOption{mathletters}\n"
+                      .."  \\usepackage[utf8x]{inputenc}\n"
+                      .."\\fi\\fi\n\n"
                       .."\\usepackage{"..pgf.STYLE_FILE_BASENAME.."}\n"
                       .."\\pagestyle{empty}\n"
                       .."\\usepackage[active,tightpage]{preview}\n"
@@ -1382,6 +1394,9 @@ gfx.TEXT_ANCHOR = {
 gfx.HEAD_STR = {"", "->", "<-", "<->", 
                 ",draw opacity=0", "->,draw opacity=0", "<-,draw opacity=0", "<->,draw opacity=0",
 		""}
+gfx.HEAD90_STR = {"", "-|", "|-", "|-|",
+                ",draw opacity=0", "-|,draw opacity=0", "|-,draw opacity=0", "|-|,draw opacity=0",
+		""}
 
 -- conversion factors in `cm'
 gfx.units = {
@@ -1477,9 +1492,9 @@ gfx.adjust_plotbox = function()
       gfx.scalex = gfx.scalex*gfx.opt.plotsize_x * pgf.DEFAULT_RESOLUTION/(t.xright - t.xleft)
       gfx.scaley = gfx.scaley*gfx.opt.plotsize_y * pgf.DEFAULT_RESOLUTION/(t.ytop - t.ybot)
     else
-      -- could not determin a valid bounding box, so keep using the
+      -- could not determine a valid bounding box, so keep using the
       -- plotsize as the canvas size
-      gp.term_out("WARNING: PGF/TikZ Terminal: `plotsize' option used, but I could not determin the plot area!\n")
+      gp.term_out("WARNING: PGF/TikZ Terminal: `plotsize' option used, but I could not determine the plot area!\n")
     end
   elseif not gfx.opt.tightboundingbox then
     if gfx.opt.clip then
@@ -1625,7 +1640,7 @@ gfx.format_color = function(ctype, val)
   elseif ctype == 'RGBA' then
     c = string.format("rgb color={%.3f,%.3f,%.3f}", val[1], val[2], val[3])
   elseif ctype == 'GRAY' then
-    c = string.format("color=black!%i", 100*val[1]+0.5)
+    c = string.format("color=black!%i", math.floor(100*val[1]+0.5))
   end
   return c
 end
@@ -1819,6 +1834,8 @@ term.options = function(opt_str, initial, t_count)
     elseif almost_equals(o_next, "c$olor") or almost_equals(o_next, "c$olour") then
       -- colored lines
       gfx.opt.lines_colored = true
+    elseif almost_equals(o_next, "butt") then
+      gfx.opt.butt = true
     elseif almost_equals(o_next, "notime$stamp") then
       -- omit output of the timestamp
       gfx.opt.notimestamp = true
@@ -1850,7 +1867,7 @@ term.options = function(opt_str, initial, t_count)
       get_next_token()
       gfx.opt.plotsize_x, gfx.opt.plotsize_y = get_two_sizes(o_next)
       if not gfx.opt.plotsize_x then
-        gp.int_error(t_count, string.format("error: two comma seperated lengths expected, got `%s'.", o_next))
+        gp.int_error(t_count, string.format("error: two comma separated lengths expected, got `%s'.", o_next))
       end
       gfx.opt.set_plotsize = true
       term_opt_size = string.format("plotsize %s,%s ", gfx.opt.plotsize_x, gfx.opt.plotsize_y)
@@ -1862,7 +1879,7 @@ term.options = function(opt_str, initial, t_count)
       get_next_token()
       local plotsize_x, plotsize_y = get_two_sizes(o_next)
       if not plotsize_x then
-        gp.int_error(t_count, string.format("error: two comma seperated lengths expected, got `%s'.", o_next))
+        gp.int_error(t_count, string.format("error: two comma separated lengths expected, got `%s'.", o_next))
       end
       gfx.opt.set_plotsize = false
       term_opt_size = string.format("size %s,%s ", plotsize_x, plotsize_y)
@@ -1872,13 +1889,13 @@ term.options = function(opt_str, initial, t_count)
       get_next_token()
       charsize_h, charsize_v = get_two_sizes(o_next)
       if not charsize_h then
-        gp.int_error(t_count, string.format("error: two comma seperated lengths expected, got `%s'.", o_next))
+        gp.int_error(t_count, string.format("error: two comma separated lengths expected, got `%s'.", o_next))
       end
     elseif almost_equals(o_next, "sc$ale") then
       get_next_token()
       local xscale, yscale = get_two_sizes(o_next)
       if not xscale then
-        gp.int_error(t_count, string.format("error: two comma seperated numbers expected, got `%s'.", o_next))
+        gp.int_error(t_count, string.format("error: two comma separated numbers expected, got `%s'.", o_next))
       end
       term_opt_scale = string.format("scale %s,%s ",xscale, yscale)
       term.xmax = term.xmax * xscale
@@ -1890,7 +1907,7 @@ term.options = function(opt_str, initial, t_count)
       for i = 1,#args do
         args[i] = tonumber(args[i])
         if args[i] == nil then
-          gp.int_error(t_count, string.format("error: list of comma seperated numbers expected, got `%s'.", o_next))
+          gp.int_error(t_count, string.format("error: list of comma separated numbers expected, got `%s'.", o_next))
         end
         args[i] = args[i] - 1
       end
@@ -1926,7 +1943,7 @@ term.options = function(opt_str, initial, t_count)
       -- produce full tex document
       gfx.opt.full_doc = true
     elseif almost_equals(o_next, "create$style") then
-      -- creates the coresponding LaTeX style from the script
+      -- creates the corresponding LaTeX style from the script
       pgf.create_style()
     elseif almost_equals(o_next, "backg$round") then
       -- set background color
@@ -1951,7 +1968,7 @@ term.options = function(opt_str, initial, t_count)
       get_next_token()
       if o_type == 'number' then
         fontscale = tonumber(o_next)
-        if fontscale < 0 then
+        if fontscale <= 0 then
           fontscale = 1.0
         end
       else
@@ -2092,6 +2109,7 @@ term.options = function(opt_str, initial, t_count)
   tf(gfx.opt.set_origin, 'originreset', 'nooriginreset')
   tf(gfx.opt.direct_image, 'bitmap', 'nobitmap')
   tf(gfx.opt.cmykimage, 'cmykimage', 'rgbimage')
+  tf(gfx.opt.butt,'butt', '')
   tf(gfx.opt.clip, 'clip', 'noclip')
   tf(gfx.opt.tightboundingbox, 'tightboundingbox', 'notightboundingbox')
   if term.external_images ~= nil then
@@ -2250,7 +2268,11 @@ term.arrow = function(sx, sy, ex, ey, head, length, angle, backangle, filled)
     gfx.check_linetype()
     gfx.check_dashtype()
     gfx.check_linewidth()
-    pgf.draw_arrow({{sx,sy},{ex,ey}}, gfx.HEAD_STR[head+1], headstyle)
+    if angle == 90 then
+      pgf.draw_arrow({{sx,sy},{ex,ey}}, gfx.HEAD90_STR[head+1], headstyle)
+    else
+      pgf.draw_arrow({{sx,sy},{ex,ey}}, gfx.HEAD_STR[head+1], headstyle)
+    end
     return 1
   end
 end
@@ -2377,7 +2399,7 @@ term.filled_polygon = function(style, fillpar, t)
       color = 'gpbgfillcolor'
       saturation = 100
       opacity = 100
-  elseif style == 'DEFAULT' or style == 'OPAQUE' then -- FIXME: not shure about the opaque style
+  elseif style == 'DEFAULT' or style == 'OPAQUE' then -- FIXME: not sure about the opaque style
       pattern = ''
       color = gfx.color
       saturation = 100
